package cn.com.brilliance.doc.controller;

import cn.com.brilliance.doc.config.DocBean;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;


/**
 * @Author weiguanying
 * @Date 2023-02-20
 * download pdf file
 */
@RestController
@RequestMapping("/api")
@Slf4j
public class PdfDownLoadController {
    private static final Logger logger = LoggerFactory.getLogger(PdfDownLoadController.class);

    // 注入配置类
    @Autowired
    private DocBean docBean;

    @RequestMapping(value = "/downLoadPdf", method = RequestMethod.POST)
    public void downLoad(@RequestParam(value = "fileName") String fileName, HttpServletResponse response) throws IOException {
        //输入处理流
        InputStream inputStream = null;
        //输出处理流
        OutputStream outputStream = null;
        long start = System.currentTimeMillis();
        File file = new File(docBean.getFilePath() + fileName);

        if (!file.exists()) {//如果文件不存在
            logger.error("下载的文件不存在");
            return;
        }
        try {
            //定义文件输入流与输出流
            inputStream = new BufferedInputStream(new FileInputStream(file));
            outputStream = new BufferedOutputStream(response.getOutputStream());

            logger.debug("设置response携带信息");
            logger.debug("文件的大小:" + file.length());
            response.reset();
            // 设置在下载框默认显示的文件名
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            //写明文件大小
            response.addHeader("Content-Length", "" + file.length());
            // 指明response的返回对象是文件流
            response.setContentType("application/octet-stream");
            //response.setContentType("multipart/form-data");

            logger.debug("开始读取文件流");
            //返回从该输入流中可以读取（或跳过）的字节数的估计值，而不会被下一次调用此输入流的方法阻塞
            byte[] bytes = new byte[inputStream.available()];
            //将pdf的内容写入bytes中
            inputStream.read(bytes);
            //将bytes中的内容写入
            outputStream.write(bytes);
            //刷新输出流，否则不会写出数据
            outputStream.flush();
            logger.debug("======文件下载处理完成==========");
        } catch (FileNotFoundException e) {
            logger.error("转换的pad文件未找到" + e.getMessage());
            return;
        } catch (IOException e) {
            logger.error("文件读写异常" + e.getMessage());
            return;
        } finally {//关闭输入流、输出流
            try {
                if (inputStream != null) {
                    inputStream.close();
                    logger.debug("输入流关闭成功！");
                }
                if (outputStream != null) {
                    outputStream.close();
                    logger.debug("输出流关闭成功！");
                }
                long end = System.currentTimeMillis();
                logger.debug("文件流处理时间" + (end - start));

            } catch (IOException e) {
                logger.error("关闭处理流出现异常" + e.getMessage());
                return;
            }
        }
    }
}