package cn.com.brilliance.doc.service.impl;

import cn.com.brilliance.doc.config.DocBean;
import cn.com.brilliance.doc.service.WordTemplateService;
import com.deepoove.poi.XWPFTemplate;
import fr.opensagres.xdocreport.document.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import fr.opensagres.xdocreport.document.IXDocReport;
import fr.opensagres.xdocreport.document.registry.XDocReportRegistry;
import fr.opensagres.xdocreport.template.IContext;
import fr.opensagres.xdocreport.template.TemplateEngineKind;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sun.misc.BASE64Encoder;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @author weiguanying
 * @date 2023-02-20
 * @description word-docx模板生成docx并返回文件名称
 */
@Service
public class WordTemplateServiceImpl implements WordTemplateService {
    private static final Logger logger = LoggerFactory.getLogger(WordTemplateServiceImpl.class);

    @Autowired
    private DocBean docBean;

    @Override
    public String docxTemplateToDocx(String templateName) {
        InputStream ins = null;
        OutputStream out = null;
        String templatesPath = docBean.getTemplatesPath() + templateName;
        try {

            //获取docx模板，模板存放路径 xxx.docx
            ins = new FileInputStream(templatesPath);
            //注册xdocreport实例并加载FreeMarker模板引擎
            IXDocReport report = XDocReportRegistry.getRegistry().loadReport(ins, TemplateEngineKind.Freemarker);
            //创建xdocreport上下文对象，用于存放具体数据
            IContext context = report.createContext();

            //创建要替换的文本变量
            context.put("name", "李梅");
            context.put("age", "20");
            context.put("school", "武汉大学");

            String FileId = UUID.randomUUID().toString();

            String newFileName = FileId + "-" + templateName;

            //输出到本地目录
            String filePath = docBean.getFilePath() + newFileName;


            out = new FileOutputStream(filePath);
            // 将前端参数写入文件输出流并保存为docx文件
            report.process(context, out);
            // 返回根据docx模板生产得docx文件
            return newFileName;

        } catch (Exception e) {
            logger.info("生成word-docx发生异常", e);
        } finally {
            try {
                if (ins != null) {
                    ins.close();
                }
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                logger.info("文件流关闭失败", e);
            }
        }
        return null;
    }

    /**
     * 基于poi-tl模板引擎生成docx模板文件为docx
     *
     * @param paramMap 模板文件替换参数
     * @return
     */
    @Override
    public String poiCreateWord(Map<String, Object> paramMap) {
        String templatePath = docBean.getTemplatesPath() + paramMap.get("templateName");
        String targetPath = docBean.getFilePath() + paramMap.get("fileName");
        // 读取模板渲染参数
        XWPFTemplate template = XWPFTemplate.compile(templatePath).render(paramMap);
        try {
            // 将模板参数写入路径
            template.writeToFile(targetPath);
            template.close();
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("模板文件解析失败");
        }
        return targetPath;
    }

    /**
     * 读取模板文件并生成docx返回base码预览和下载
     * @date 2023-03-21
     * @author weiguanying
     * @param paramMap
     * @param response
     * @return
     * @throws IOException
     */
    @Override
    public String wordConvertAndPreview(Map<String, Object> paramMap, HttpServletResponse response) throws IOException {
        File sourceFile = new File(docBean.getTemplatesPath() + paramMap.get("templateName"));
        //在模板文件中任意表格位置填充数据
        Map<String, Object> data = new HashMap();
        data.put("title", paramMap.get("title"));

        //浏览器端下载
        response.setCharacterEncoding("utf-8");
        response.setContentType("application/msword" + ";charset=UTF-8");
        String fileName = "preview" + ".docx";
        response.setHeader("Content-Disposition", "attachment;filename="
                .concat(String.valueOf(URLEncoder.encode(fileName, "UTF-8"))));
        response.flushBuffer();
        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(sourceFile));
        //创建输出流
        OutputStream os = response.getOutputStream();
        //最终编译渲染并输出
        XWPFTemplate.compile(sourceFile).render(data).writeAndClose(os);
        // 创建字节输出流
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int i = bis.read(buffer);
        while (i != -1) {
            os.write(buffer, 0, i);
            outputStream.write(buffer, 0, i);
            i = bis.read(buffer);
        }
        // 读取输出流字节
        byte[] bytes = outputStream.toByteArray();
        // 转BASE64编码字符串
        BASE64Encoder encoder = new BASE64Encoder();
        String base64String = encoder.encodeBuffer(bytes).trim();
        if (bis != null) {
            bis.close();
        }
        if (os != null) {
            os.close();
        }
        if (outputStream != null) {
            outputStream.close();
        }
        System.out.println(base64String);
        // response.se
        return base64String;
        // return base64String.replaceAll("\n", "").replaceAll("\r", "");

    }

}

